import 'dart:convert';
import 'dart:ffi';
import 'package:ffi/ffi.dart';
import 'dart:typed_data';
import 'array_utils.dart';

/// Thin wrapper arount a [Pointer<T>] and its associated typed list [L]
class Array<T extends NativeType, L extends List<int>> {
  /// Pointer to array
  final Pointer<T> ptr;

  /// View into array
  L get view => _view;
  L _view;

  Array.fromPtr(this.ptr, this._view);

  /// Allocates Array of [length].
  /// [getView] returns the typed list of [Pointer<T>]
  Array(int length, L Function(Pointer<T>, int) getView)
      : ptr = allocate<T>(count: length) {
    _view = getView(ptr, length);
  }
  Array.fromList(L list, L Function(Pointer<T>, int) getView)
      : ptr = allocate<T>(count: list.length) {
    _view = getView(ptr, list.length);
    _view.setAll(0, list);
  }

  void delete() {
    free(ptr);
  }
}

class Uint8Array extends Array<Uint8, Uint8List> {
  Uint8Array.fromPtr(Pointer<Uint8> ptr, Uint8List view)
      : super.fromPtr(ptr, view);
  Uint8Array(int length) : super(length, getUint8View);
  Uint8Array.fromStr(String str) : super(str.length, getUint8View) {
    final utf8enc = utf8.encode(str);
    _view.setAll(0, utf8enc);
  }
  Uint8Array.fromUint8(Uint8List list) : super.fromList(list, getUint8View);
}
