import 'package:ffi_helper/ffi_helper.dart';
import 'package:test/test.dart';
import 'dart:ffi';

void main() {
  test('Get slices of Arena', () {
    final arena = Uint8Arena(10);
    expect(arena.freeSpace, 10);
    final slice1 = arena.newArray(5);
    expect(slice1.view.length, 5);
    expect(arena.freeSpace, 5);
    final slice2 = arena.newArray(5);
    expect(slice2.view.length, 5);
    expect(arena.freeSpace, 0);
    arena.delete();
  });

  test('write / read slice of Arena', () {
    final arena = Uint8Arena(10);
    final arr = arena.newArray(5);
    final arrView = arr.ptr.asTypedList(5)..setAll(0, const [1, 2, 3, 4, 5]);
    expect(arrView, equals(arr.view));

    final arr2 = arena.newArray(3);
    final arr2View = arr2.ptr.asTypedList(3)..setAll(0, const [1, 2, 3]);
    expect(arr2View, equals(arr2.view));

    expect(arena.freeSpace, 2);
    arena.delete();
  });

  test('throws RangeError when overflow', () {
    final arena = Uint8Arena(8);
    arena.newArray(5);
    expect(() => arena.newArray(5), throwsRangeError);
    arena.delete();
  });
}
